/*
	IntToWavCmd.cc

	Will Eighteen, 2006: will@opaline.demon.co.uk
*/

#include "IntToWavCmd.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>
#include "OpResources.h"


IntToWavCmd::IntToWavCmd ( char *name, int active, char *intFile, char *wavFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, intFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, wavFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_totalSamples = 0;
	_pWave = NULL;
	_percentDone = 0;
	_error = 0;
}


IntToWavCmd::~IntToWavCmd()
{	
	// empty
}
	

void IntToWavCmd::doCommand()
{
	int percent;
	int _numInts = 250;	/* max ints processed in a 'doCommand' call */
	int error = 0;
	char text[5];

	if ( _done == True )
		return;
	
	if ( !_once )
	{
		// can we open source for read?
		FileAccess *src = new FileAccess ( _srcFile, "r" );
		
		if ( src->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, src->error() );
			msg [ sizeof (msg ) - 1 ] = 0;			
			messageDialogManager->post ( msg );
		}
		
		delete src;
		
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );
		
		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof (msg ) - 1 ] = 0;			
			messageDialogManager->post ( msg );
		}
		
		delete dst;
		
		if ( error )
		{
			_done = True;
			_error = error;
			return;
		}

		_pWave = new Wave;
		_pWave->FillHeader ( _channels, _samplesPerSec, _bitsPerSample, 0 );

		if ( !_pWave->SaveHeader ( _dstFile ) )
		{
			char msg[BUFSIZE];
	  
	   		strncpy ( msg, _pWave->name(), sizeof ( msg ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			strncat ( msg, "\nWave::SaveHeader failed.", sizeof ( msg ) - strlen ( msg ) - 1 );
			messageDialogManager->post ( msg );
			delete _pWave;
			_done = True;
			_error++;
			return;
		}
		
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		FileAccess *out = new FileAccess ( _dstFile, "w"  );
		
		_in = in;
		_out = out;

		_in->elementSize ( sizeof ( int ) );
		_out->elementSize ( mixwaveResources->bitsPerSample() / 8 );
		_in->numElements ( 1 );
		_out->numElements ( 1 );

		_out->filePos ( _pWave->HeaderSize() );	
		updateTimeout ( ( int ) ( _in->fileSize() / ( _numInts * sizeof ( int ) ) ) + 1 );
		_once = 1;
	}
	
	if ( _in->fileSize() / sizeof ( int ) < _numInts )
		_numInts = ( int ) ( _in->fileSize() / sizeof ( int ) );

	char buf[8];
	int count;

	if ( _in->filePos() < _in->fileSize() )
	{
		for ( count = 0; count < _numInts; count++ )
		{
			_in->readF ( &_iValue );
			buf[0] =  _iValue&0x00ff;
			buf[1] =  _iValue>>8;			// assumes 16-bit
			_out->writeF ( &buf );
			_totalSamples++;
		}

		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}

	if ( _in->filePos() == _in->fileSize() )
	{
		_pWave->FillHeader ( _channels, _samplesPerSec, _bitsPerSample, _totalSamples );
		_pWave->ChangeHeader ( _dstFile );
		delete _in;
		delete _out;
		delete _pWave;
		_done = True;
	}
}


void IntToWavCmd::setWaveDefaults ( long samplesPerSec, short bitsPerSample, short channels )
{
	_samplesPerSec = samplesPerSec;
	_bitsPerSample = bitsPerSample;
	_channels = channels;
}

