/*
 BusyPixmapBar.cc
 Will 18, 2000; will@opaline.demon.co.uk

 Adapted from 'BusyPixmap'
 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/

#include "BusyPixmapBar.h"
#include <Xm/Xm.h>

#define NUMPIXMAPS 11
#define XPIXMAPSIZE 120
#define YPIXMAPSIZE 12


BusyPixmap::BusyPixmap ( Widget w ) : PixmapCycler ( NUMPIXMAPS, XPIXMAPSIZE, YPIXMAPSIZE )
{
	_w = w;
}


void BusyPixmap::createPixmaps()
{
	int progress, delta, i;
	XGCValues gcv;
	
	// creat a gc based on colours of given widget
	XtVaGetValues ( _w, XmNforeground, &gcv.foreground,
					XmNbackground, &gcv.background, NULL );
					
	_gc = XtGetGC ( _w, GCForeground | GCBackground, &gcv );
	
	XtVaGetValues ( _w, XmNforeground, &gcv.background,
					XmNbackground, &gcv.foreground, NULL );
					
	_inverseGC = XtGetGC ( _w, GCForeground | GCBackground, &gcv );
	
	// define starting increment and size of 'done' block
	delta = ( int ) XPIXMAPSIZE / ( NUMPIXMAPS - 1);	// get bar width percentage
	progress = 0;

	for ( i = 0; i < NUMPIXMAPS; i++ )
	{
		_pixmapList[i] = createBusyPixmap ( progress, YPIXMAPSIZE );
		progress += delta;
	}
	
	XtReleaseGC ( _w, _gc );
	XtReleaseGC ( _w, _inverseGC );				
}


Pixmap BusyPixmap::createBusyPixmap ( int width, int height )
{
	Pixmap pm;
	// use root window to create pixmap in (widget may be unrealized or gadget)
	pm = XCreatePixmap ( XtDisplay ( _w ), RootWindowOfScreen ( XtScreen ( _w ) ),
						 _width, _height,
						 DefaultDepthOfScreen ( XtScreen ( _w ) ) );
						 
	XFillRectangle ( XtDisplay ( _w ), pm, _inverseGC, 0, 0, _width, _height );
	XFillRectangle ( XtDisplay ( _w ), pm, _gc, 0, 0, width, height );
	return ( pm );
}


/*
  Notes:
  
  This class creates a sequence of bitmaps consisting of a bar with progressive
  filled sections.
  
  createPixmaps() creates two GCs, the first corresponding to the fg and bg
  colours of the widget passed to the constructor, the second being the inverse
  of the first. createBusyPixmap() uses these when creating and rendering each
  pixmap. createPixmaps() calls createBusyPixmap() in a loop.
  
  The BusyPixmap constructor requires a widget used to create a graphics context;
  the colours used in the pixmaps match the foreground and background colours of
  this widget. An array of (11 pixmaps 120x30) pixels are created.
*/
