// MenuApp.cc

#include "MenuApp.h"
#include <QuitCmd.h>
#include <ManageCmd.h>
#include <IconifyCmd.h>
#include <NOPCmd.h>
#include "MenuWindowApp.h"

MenuApp *menuApp = new MenuApp ( "menuDemo" );
MainWindow *window1 = new MenuWindowApp ( "Window1" );
MainWindow *window2 = new MenuWindowApp ( "Window2" );
MainWindow *window3 = new MenuWindowApp ( "Window3" );


MenuApp::MenuApp ( const char *name ) : Application ( name )
{
	// create Application-wide commands appearing in all menus
	_quit = new QuitCmd ( "Quit" );
	_manage = new ManageCmd ( "Open" );
	_iconify = new IconifyCmd ( "Icon" );
	
	// create 3 NOPCmd objects to show dependencies and support for multiple
	// interfaces to a single Cmd
	_x = new NOPCmd ( "X" );
	_y = new NOPCmd ( "Y" );
	_z = new NOPCmd ( "Z", FALSE );
	
	// specify relationships between X, Y and Z commands
	// activating one deactivates itself and activates the others
	_y->addToActivationList ( _x );
	_y->addToActivationList ( _z );
	_y->addToDeactivationList ( _y );
	
	_x->addToActivationList ( _y );
	_x->addToActivationList ( _z );
	_x->addToDeactivationList ( _x );
	
	_z->addToActivationList ( _x );
	_z->addToActivationList ( _y );
	_z->addToDeactivationList ( _z );
}

MenuApp::~MenuApp()
{
	delete _quit;
	delete _manage;
	delete _iconify;
	delete _x;
	delete _y;
	delete _z;
}


/*
  Notes:
  
  The constructor creates Cmd objects for the first two menu panes appearing in
  each window. These Cmd objects cannot be added to a menu yet; the menu must be
  constructed in MenuWindowApp. However, since there is only one instance of each
  of the two classes in the whole application, the Quit, Open and Icon objects
  are instantiated in the MenuApp constructor.
  
  The constructor also creates three NOPCmd objects used in menu panes of all
  three windows. Although menus are constructed in MenuApp, we can specify
  relations between objects when they are created. The application determines
  which commands need to be (in)active. The Cmd objects handle the interface
  components, regardless of how many interfaces are assocaited with a command.
  
  Here, X and Y are initially active and Z inactive ("FALSE"). A '2 of 3' toggle
  behaviour is implemented.
*/
	
